import os
import shutil
import unittest
import tempfile
from pathlib import Path

import jinjabread


def print_tree(dir):
    for path in Path(dir).rglob("*"):
        print(path.relative_to(dir).as_posix())


def list_files(dir):
    return [
        path.relative_to(dir).as_posix()
        for path in Path(dir).rglob("*")
        if path.is_file()
    ]


class SiteTest(unittest.TestCase):

    def setUp(self):
        self.project_dir = Path(tempfile.mkdtemp())
        os.chdir(self.project_dir)
        self.addCleanup(shutil.rmtree, self.project_dir)

    def test_html_content(self):
        content_path = self.project_dir / "content" / "home.html"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        with content_path.open("w") as file:
            file.write("<h1>Hello, World{# This is a comment #}</h1>")

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ],
        )
        site(prettify_html=False)

        self.assertEqual("<h1>Hello, World</h1>", Path("public/home.html").read_text())

    def test_html_content_extends_layout(self):
        content_path = self.project_dir / "content" / "home.html"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        with content_path.open("w") as file:
            file.write(
                "{% extends 'base.html' %}{% block body %}<p>Blah blah blah</p>{% endblock body %}"
            )
        layout_path = self.project_dir / "layouts" / "base.html"
        layout_path.parent.mkdir(parents=True, exist_ok=True)
        with layout_path.open("w") as file:
            file.write("<h1>Hello, World</h1>{% block body %}{% endblock body %}")

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ]
        )
        site(prettify_html=False)

        self.assertEqual(
            "<h1>Hello, World</h1><p>Blah blah blah</p>",
            Path("public/home.html").read_text(),
        )

    def test_html_content_includes_layout(self):
        content_path = self.project_dir / "content" / "home.html"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        with content_path.open("w") as file:
            file.write("<h1>Hello, World</h1>{% include 'message.txt' %}")
        layout_path = self.project_dir / "layouts" / "message.txt"
        layout_path.parent.mkdir(parents=True, exist_ok=True)
        with layout_path.open("w") as file:
            file.write("<p>Blah blah blah</p>")

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ]
        )
        site(prettify_html=False)

        self.assertEqual(
            "<h1>Hello, World</h1><p>Blah blah blah</p>",
            Path("public/home.html").read_text(),
        )
        self.assertFalse(Path("public/message.txt").exists())

    def test_html_content_includes_content(self):
        content_path = self.project_dir / "content" / "home.html"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        with content_path.open("w") as file:
            file.write("<h1>Hello, World</h1>{% include 'message.txt' %}")
        content_path2 = self.project_dir / "content" / "message.txt"
        content_path2.parent.mkdir(parents=True, exist_ok=True)
        with content_path2.open("w") as file:
            file.write("<p>Blah blah blah</p>")

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ]
        )
        site(prettify_html=False)

        self.assertEqual(
            "<h1>Hello, World</h1><p>Blah blah blah</p>",
            Path("public/home.html").read_text(),
        )
        self.assertTrue(Path("public/message.txt").exists())

    def test_text_content(self):
        content_path = self.project_dir / "content" / "home.txt"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        with content_path.open("w") as file:
            file.write("Hello, World{# This is a comment #}")

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ]
        )
        site()

        self.assertEqual("Hello, World", Path("public/home.txt").read_text())

    def test_markdown_content(self):
        content_path = self.project_dir / "content" / "home.md"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        with content_path.open("w") as file:
            file.write(
                """
---
author: John
---
# Hello, World! {# This is a comment #}
This is my story.
""".lstrip()
            )
        layout_path = self.project_dir / "layouts" / "base.html"
        layout_path.parent.mkdir(parents=True, exist_ok=True)
        with layout_path.open("w") as file:
            file.write(
                "{{ content }}{# This is another comment #}<p>Written by {{ author }}.</p>"
            )

        site = jinjabread.Site(
            pages=[
                jinjabread.MarkdownPage(layout_name="base.html"),
            ]
        )
        site()

        self.assertEqual(
            """
<h1>
  Hello, World!
</h1>
<p>
  This is my story.
</p>
<p>
  Written by John.
</p>
""".lstrip(),
            Path("public/home.html").read_text(),
        )

    def test_copy_static_content(self):
        content_path = self.project_dir / "content" / "dummy.jpg"
        content_path.parent.mkdir(parents=True, exist_ok=True)
        content_path.touch()

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ]
        )
        site()

        self.assertTrue(Path("public/dummy.jpg").exists())

    def test_copy_static_directory(self):
        static_path = self.project_dir / "static" / "dummy.jpg"
        static_path.parent.mkdir(parents=True, exist_ok=True)
        static_path.touch()

        site = jinjabread.Site(
            pages=[
                jinjabread.Page(),
            ]
        )
        site()

        self.assertTrue(Path("public/static/dummy.jpg").exists())



import argparse
from . import create_new_project, build, serve

def main(action, **options):
    match action:
        case "new":
            create_new_project(options["site_name"])

        case "build":
            build(options["project_dir"])

        case "serve":
            serve(options["project_dir"])

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="jinjabread",
        description="A Python-based static site generator using Jinja templates.",
    )
    subparsers = parser.add_subparsers(required=True, dest="action")
    new_site_parser = subparsers.add_parser("new", help="Create new site.")
    new_site_parser.add_argument("site_name", help="Your site name.")
    serve_parser = subparsers.add_parser("serve", help="Run development web server.")
    serve_parser.add_argument("project_dir", help="The site directory.")
    build_parser = subparsers.add_parser("build", help="Build site.")
    build_parser.add_argument("project_dir", help="The site directory.")
    args = parser.parse_args()

    main(**vars(args))



from pathlib import Path
import re
import shutil

import bs4
from jinja2 import Environment, FileSystemLoader
import markdown


class Site:

    def __init__(
        self,
        *,
        pages,
        project_dir=".",
        content_dir="content",
        layouts_dir="layouts",
        static_dir="static",
        output_dir="public"
    ):
        self.project_dir = Path(project_dir)
        if not self.project_dir.exists():
            raise FileNotFoundError("The project directory doesn't exist.")
        self.content_dir = self.project_dir / content_dir
        self.layouts_dir = self.project_dir / layouts_dir
        self.static_dir = self.project_dir / static_dir
        self.output_dir = self.project_dir / output_dir
        self.env = Environment(
            loader=FileSystemLoader(searchpath=[self.layouts_dir, self.content_dir])
        )
        self.pages = pages

    def __call__(self, prettify_html=True):
        self.prettify_html = prettify_html
        self.generate()

    def render_template(self, template_name, **context):
        template = self.env.get_template(template_name)
        return template.render(context)

    def match_page(self, path):
        for page in self.pages:
            if re.match(page.path_pattern, path.name):
                return page

    def generate(self):
        for content_path in self.content_dir.rglob("*"):
            if content_path.is_dir():
                continue
            if content_path.suffix not in [".html", ".md", ".txt"]:
                output_path = self.output_dir / content_path.relative_to(
                    self.content_dir
                )
                output_path.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy(content_path, output_path)
                continue
            page = self.match_page(content_path)
            if not page:
                continue
            page(self, content_path, self.prettify_html)

        if self.static_dir.exists():
            shutil.copytree(
                self.static_dir,
                self.output_dir / self.static_dir.name,
                dirs_exist_ok=True,
            )


class Page:
    path_pattern = r".*"

    def __call__(self, site, content_path, prettify_html=True):
        self.site = site
        self.content_path = content_path
        self.output_path = self.get_output_path()
        self.prettify_html = prettify_html
        self.generate()

    def get_output_path(self):
        return self.site.output_dir / self.content_path.relative_to(
            self.site.content_dir
        )

    def get_template_name(self):
        return self.content_path.relative_to(self.site.content_dir).as_posix()

    def get_context(self):
        path = "/" + self.output_path.relative_to(self.site.output_dir).as_posix()
        return {
            "path": path,
        }

    def render(self):
        template_name = self.get_template_name()
        text = self.site.render_template(template_name, **self.get_context())
        if self.prettify_html and self.output_path.suffix == ".html":
            return _prettify_html(text)
        return text

    def generate(self):
        text = self.render()
        self.output_path.parent.mkdir(parents=True, exist_ok=True)
        with self.output_path.open("w") as file:
            file.write(text)


class MarkdownPage(Page):
    path_pattern = r".*\.md"

    def __init__(self, *, layout_name):
        self.layout_name = layout_name
        self.markdown = markdown.Markdown(extensions=['full_yaml_metadata'])

    def get_output_path(self):
        return super().get_output_path().with_suffix(Path(self.layout_name).suffix)

    def get_template_name(self):
        return self.layout_name

    def get_context(self):
        context = super().get_context()
        text = self.site.render_template(
            self.content_path.relative_to(self.site.content_dir).as_posix(), **context
        )
        context["content"] = self.markdown.convert(text)
        metadata = self.markdown.Meta
        if metadata:
            context.update(**metadata)
        return context


def create_new_project(site_name):
    project_path = Path(site_name)
    content_path = project_path / "content" / "index.md"
    content_path.parent.mkdir(parents=True)
    with content_path.open("w") as file:
        file.write(
            """
---
author: me
---
# Hello, World!
This is my new website.
""".lstrip()
        )
    layout_path = project_path / "layouts" / "base.html"
    layout_path.parent.mkdir(parents=True)
    with layout_path.open("w") as file:
        file.write(
            """
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Jinjabread Website</title>
</head>
<body>
  {{ content }}
  <p>Created by {{ author }}.<p>
</body>
</html>
""".lstrip()
        )

def _prettify_html(text):
    soup = bs4.BeautifulSoup(text, "html.parser")
    return soup.prettify(formatter=bs4.formatter.HTMLFormatter(indent=2))



import itertools
import mimetypes
from pathlib import Path
from werkzeug.serving import run_simple
from werkzeug.wrappers import Request, Response
from werkzeug.utils import redirect
from . import build


class App:

    def __init__(self, project_dir):
        self.public_dir = Path(project_dir) / "public"

    def dispatch_request(self, request):
        path = Path(request.path)
        if path.suffix == ".html":
            return redirect(path.with_suffix(""))

        file_path = self.public_dir / path.relative_to("/")
        if (not file_path.exists() or file_path.is_dir()) and file_path.with_suffix(
            ".html"
        ).exists():
            file_path = file_path.with_suffix(".html")
        elif file_path.is_dir():
            file_path /= "index.html"

        try:
            with file_path.open("rb") as file:
                mimetype, _ = mimetypes.guess_type(file_path.name)
                return Response(
                    file.read(),
                    status=200,
                    mimetype=mimetype,
                )
        except FileNotFoundError:
            return Response(f"File Not Found: {file_path}", status=404)

    def wsgi_app(self, environ, start_response):
        request = Request(environ)
        response = self.dispatch_request(request)
        return response(environ, start_response)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)


def serve(project_dir):
    build(project_dir)
    extra_files = [
        path.as_posix()
        for path in itertools.chain(
            Path(f"{project_dir}/content").rglob("*"),
            Path(f"{project_dir}/layouts").rglob("*"),
            Path(f"{project_dir}/static").rglob("*"),
        )
        if path.is_file()
    ]
    run_simple(
        "127.0.0.1",
        8000,
        App(project_dir),
        use_reloader=True,
        extra_files=extra_files,
    )
